#!/usr/bin/env python

#  Copyright 2008, Matias Bordese

#  This file is part of PycasaDownload.

#  PycasaDownload is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PycasaDownload is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PycasaDownload. If not, see <http://www.gnu.org/licenses/>.

import os
current_abs_path = os.path.dirname(os.path.abspath(__file__))

from app.config import LARGE_THUMB, MEDIUM_THUMB, APP_ABS_PATH, IMAGES_FOLDER_PATH

import sys
sys.path.append(APP_ABS_PATH)
import thread
import urllib

TEXT_MODE = False
DEBUG = True

MAIN_SITE_URL = 'http://www.pycasa-download.com.ar'

try:
    import pygtk
    pygtk.require('2.0')
    import gtk
    import gtk.glade
    import gobject
except:
    TEXT_MODE = True

from app.PycasaDownload import PycasaDownload
from app.PycasaDownloadText import text_mode
from app.Album import Album

album_metadata = [
('title',     lambda attr: '<b>%s</b>\n' % attr),
('summary',   lambda attr: attr is not None and '<i>%s</i>\n\n' % attr or '\n\n'),
('numphotos', lambda attr: (attr == 1 and '%s photo\n' or '%s photos\n') % attr),
('published', lambda attr: 'Published %s' % attr)
]

photo_metadata = [
('title',     lambda attr: '<b>%s</b>\n' % attr),
('summary',   lambda attr: attr is not None and '<i>%s</i>\n\n' % attr or '\n\n'),
('weight',    lambda attr: '%s bytes\n' % attr),
('size',      lambda func: '%s x %s\n' % func()),
('published', lambda attr: 'Published %s' % attr)
]

path_to_icon = os.path.join(IMAGES_FOLDER_PATH, 'icon.png')
path_to_logo = os.path.join(IMAGES_FOLDER_PATH, 'logo-50.png')

class PycasaDownloadGUI:
    """This is a GUI for the PycasaDownload class"""

    def __init__(self):

        #Set the Glade file
        self.gladefile = os.path.join(APP_ABS_PATH, 'pycasa_download.glade')
        self.wTree = gtk.glade.XML(self.gladefile)

        #Init the PycasaDownload object
        self.pd = PycasaDownload()

        #Get the widgets in the window
        self.window = self.wTree.get_widget('MainWindow')
        self.window.set_icon_from_file(path_to_icon)
        gtk.window_set_default_icon_from_file(path_to_icon)

        self.in_user = self.wTree.get_widget('txtUsername')
        self.progress = self.wTree.get_widget('prgDownload')

        self.btnDownload = self.wTree.get_widget('btnDownload')
        self.btnCancel = self.wTree.get_widget('btnCancel')
        self.btnUpdate = self.wTree.get_widget('btnUpdate')
        self.hbxUsername = self.wTree.get_widget('hbxUsername')

        self.lblPreview = self.wTree.get_widget('lblPreview')
        self.lblPreview.set_line_wrap(True)
        self.imgPreview = self.wTree.get_widget('imgPreview')

        self.icvMain = self.wTree.get_widget('icvMain')

        self.dlgAbout = self.wTree.get_widget('dlgAbout')
        self.dlgAbout.set_name(self.pd.name)
        self.dlgAbout.set_version(str(self.pd.version))
        self.dlgAbout.set_logo(gtk.gdk.pixbuf_new_from_file(path_to_logo))
        self.dlgAbout.set_website(MAIN_SITE_URL)

        self.dlgHelp = self.wTree.get_widget('dlgHelp')
        self.ttvHelp = self.wTree.get_widget('ttvHelp')
        self.ttvHelp.get_buffer().set_text(uglyDirtyReadmeHack())

        self.dlgWarning = gtk.MessageDialog(parent=self.window, type=gtk.MESSAGE_WARNING,
                                            flags=gtk.DIALOG_MODAL, buttons=gtk.BUTTONS_CLOSE,
                                            message_format='The provided username is not valid.')
        self.dlgWarning.connect('delete-event', self.hideWidget)
        self.dlgWarning.connect('response', self.hideWidget)

        self.dlgInfo = gtk.MessageDialog(parent=self.window, type=gtk.MESSAGE_INFO,
                                            flags=gtk.DIALOG_MODAL, buttons=gtk.BUTTONS_CLOSE,
                                            message_format='New version available!.')
        self.dlgInfo.connect('delete-event', self.hideWidget)
        self.dlgInfo.connect('response', self.hideWidget)

        #Create calback dictionay and autoconnect it
        dic = {'on_btnUpdate_clicked' : self.refreshAlbums,
               'on_btnCancel_clicked' : self.cancelDownloadAlbum,
               'on_btnDownload_clicked' : self.startDownloadAlbum,
               'on_btnClose_clicked' : self.quit,
               'on_icvMain_selection_changed' : self.updatePreview,
               'on_icvMain_item_activated' : self.itemActivated,
               'on_about_activate' : lambda w: self.dlgRun(self.dlgAbout),
               'on_help_activate' : lambda w: self.dlgRun(self.dlgHelp),
               'on_check_new_version_activate': self.checkNewVersion,
               'on_MainWindow_destroy' : self.quit
              }
        self.wTree.signal_autoconnect(dic)

        self.currentAlbum = None

        #Create the model for albums (album, thumbnail, title)
        self.albumStore = gtk.ListStore(gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf, gobject.TYPE_STRING)

        #Create the model for photos (photo, thumbnail, title)
        self.photoStore = gtk.ListStore(gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf, gobject.TYPE_STRING)

        self.metadata = {self.albumStore: album_metadata, self.photoStore: photo_metadata}
        self.thumb_width = 64
        self.folder_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(IMAGES_FOLDER_PATH, 'folder.png'),
                                                                  self.thumb_width, -1)

        self.back_to_albums = BackToAlbum(self.thumb_width)

        self.icvMain.set_pixbuf_column(1)
        self.icvMain.set_markup_column(2)
        #User can download multiple albums
        self.icvMain.set_selection_mode(gtk.SELECTION_MULTIPLE)

        #The list of selected albums to download
        self.download_queue = []
        self.current = None

        #The default download directory
        self.destination = os.path.expanduser('~')

        #Is download in progress?
        self.downloadStarted = False

        self.reset()

    def addTextColumn(self, treeview, name, index, resizable=True, reorderable=True, width=-1):
        """Add a text column to the given treeview"""
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(name, renderer, text=index)
        column.set_min_width(width)
        column.set_resizable(resizable)
        column.set_reorderable(reorderable)
        treeview.append_column(column)

    def reset(self):
        self.lblPreview.set_markup('<b>Welcome to %s v%s!</b>' % (self.pd.name, self.pd.version))
        self.imgPreview.set_from_file(path_to_logo)
        self.albumStore.clear()
        self.photoStore.clear()

    def refreshAlbums(self, widget):
        """Update the albums list for the specified user"""
        #self.btnUpdate.set_sensitive(False)
        self.hbxUsername.set_sensitive(False)
        self.user = self.in_user.get_text()
        if not self.user.endswith('@gmail.com'):
            self.user = self.user + '@gmail.com'
        self.window.set_title("%s - %s's albums" % (self.pd.name, self.user))
        self.reset()
        thread.start_new(self.updateAlbums, (widget,))

    def updateAlbums(self, widget):
        """Update the albums list for the user specified"""
        self.progress.set_text('getting albums')
        albums = self.pd.getAlbumsFromUser(self.user)

        self.icvMain.set_model(self.albumStore)
        self.albumStore.clear()
        self.progress.set_text("%d albums found" % self.pd.found)
        if albums is not None:
            #Load items into tree model
            len_albums = 0
            for a in albums:
                len_albums += 1
                thumb = a.getThumbnail()
                folder_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(IMAGES_FOLDER_PATH, 'folder.png'),
                                                                     80, -1)
                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(thumb.path, 40, -1)
                assert(folder_pixbuf.get_width() >= pixbuf.get_width())
                x = folder_pixbuf.get_width() - pixbuf.get_width()
                y = folder_pixbuf.get_height() - pixbuf.get_height()
                pixbuf.composite(folder_pixbuf, x, y,
                                 pixbuf.get_width(), pixbuf.get_height(),
                                 x, y, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
                item = (a, folder_pixbuf, a.title)

                #pixbuf = gtk.gdk.pixbuf_new_from_file(thumb.path)
                #self.folder_pixbuf.composite(pixbuf, 0, 0, pixbuf.get_width(),
                #                             pixbuf.get_height(), 0, 0 , 1, 1,
                #                             gtk.gdk.INTERP_BILINEAR, 255)
                #item = (a, pixbuf, a.title)

                album_iter = self.albumStore.append(item)
        else:
            self.dlgWarning.set_title(self.user)
            if sys.platform != 'win32':
                self.dlgWarning.show()
            self.progress.set_text('Invalid username: %s' % self.user)
        #self.btnUpdate.set_sensitive(True)
        self.hbxUsername.set_sensitive(True)

    def updateThumbnails(self, widget, path):
        """Update the thumbnails view for the specifieda album"""
        self.icvMain.set_model(self.photoStore)
        self.photoStore.clear()
        self.photoStore.append((self.back_to_albums, self.back_to_albums.pixbuf, self.back_to_albums.legend))

        album_iter = self.albumStore.get_iter(path)
        album = self.albumStore.get_value(album_iter, 0)
        self.currentAlbum = album

        len_thumbs = 0
        self.progress.set_text("%d photos found" % album.numphotos)
        for t in album.photos:
            len_thumbs += 1
            thumb = t.getThumbnail(MEDIUM_THUMB)
            item = (t, gtk.gdk.pixbuf_new_from_file_at_size(thumb.path, self.thumb_width, -1), t.title)
            if self.currentAlbum is album:
                self.photoStore.append(item)
            else:
                break

    def updatePreview(self, widget):
        xs = widget.get_selected_items()
        if not xs:
            return

        self.btnDownload.set_sensitive(True)
        model = widget.get_model()
        iter = model.get_iter(xs[0])
        elem = model.get_value(iter, 0)
        if elem is self.back_to_albums:
            return

        markup = ''
        for (attr, value) in self.metadata[model]:
            markup += value(getattr(elem, attr))
        self.lblPreview.set_markup(markup)
        thumb = elem.getThumbnail(MEDIUM_THUMB)
        self.imgPreview.set_from_file(thumb.path)

    def itemActivated(self, widget, path):
        model = widget.get_model()
        item = model.get_value(model.get_iter(path), 0)
        if model is self.albumStore:
            self.window.set_title("%s - %s's content" % (self.pd.name, item.title))
            thread.start_new(self.updateThumbnails, (widget, path))
        else:
            assert(model is self.photoStore)
            if item is self.back_to_albums:
                self.icvMain.set_model(self.albumStore)
                self.currentAlbum = None
                self.progress.set_text("%d albums found" % self.pd.found)
                self.window.set_title("%s - %s's albums" % (self.pd.name, self.user))

    def hideWidget(self, widget, event):
        widget.hide()
        return True

    def dlgRun(self, dialog):
        """Show dialog"""
        dialog.run()
        dialog.hide()

    def checkNewVersion(self, *a):
        new_version = self.pd.version < urllib.urlopen('%s/getLatestVersion.php' % MAIN_SITE_URL).read().strip()
        if new_version:
            self.dlgInfo.set_markup("""There is a new version available for download!

Please go to <i>%s</i> and start enjoying the new features.""" % (MAIN_SITE_URL,))
        else:
            self.dlgInfo.set_markup("""No new version available for download.

Join us on <i>#pycasa-download</i> at irc.freenode.net and find out what we are working on!""")
        self.dlgInfo.run()

    def startDownloadAlbum(self, widget):
        """Enable the download thread to start the album download and
        manage changes in the GUI"""
        paths = self.icvMain.get_selected_items()
        store = self.icvMain.get_model()
        self.download_queue = []
        for p in paths:
            if not p is self.back_to_albums:
                album_iter = store.get_iter(p)
                self.download_queue.append(store.get_value(album_iter, 0))

        if self.download_queue:
            self.destination = self.selectBaseDirectory()
            if self.destination:
                self._startDownload()

    def downloadSelectedAlbums(self):
        """Download all selected albums"""
        c = 1
        count = len(self.download_queue)

        for i in self.download_queue:
            item_label = self.currentAlbum and 'photo' or 'album'
            self.progress.set_text("downloading %s %d of %d" % (item_label, 1.0*c, count))
            self.current = i
            self.downloadAlbum()
            self.progress.set_fraction(1.0*c/count)
            c = c + 1

        if self.downloadStarted:
            self._stopDownload('download completed')

    def downloadAlbum(self):
        """Download all the photos in the given album"""
        dest = self.createDestinationDirectory()
        self.current.download(dest)
        self.current = None

    def cancelDownloadAlbum(self, widget):
        """Cancel an album download"""
        self._stopDownload(msg='download cancelled')

    def selectBaseDirectory(self):
        """Select the root for downloading albums"""
        dialog = gtk.FileChooserDialog(parent=self.window,
                                       title='Choose the destination directory',
                                       action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                       buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                                                  gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        if self.destination:
            dialog.set_current_folder(self.destination)
        else:
            dialog.set_current_folder(os.path.expanduser('~'))
        response = dialog.run()
        if response == gtk.RESPONSE_ACCEPT:
            res = dialog.get_filename()
        else:
            res = None
        dialog.destroy()

        return res

    def createDestinationDirectory(self):
        """Create the destination directory for the album download"""
        res = self.destination
        if isinstance(self.current, Album):
            res = self.destination + '/%s' % self.current.title
            if not os.path.isdir(res):
                try:
                    os.mkdir(res);
                except:
                    res = None
        return res

    def _startDownload(self):
        """Start the album download thread and disable controls"""
        # Tell the download thread there are albums to download
        self.downloadStarted = True
        self.progress.set_text('starting download')
        self.btnDownload.set_sensitive(False)
        #self.in_user.set_sensitive(False)
        #self.btnUpdate.set_sensitive(False)
        self.hbxUsername.set_sensitive(False)
        self.icvMain.set_sensitive(False)
        self.btnCancel.set_sensitive(True)


    def _stopDownload(self, msg=''):
        """Stop the album downloading and restore the disabled controls"""
        self.btnDownload.set_sensitive(True)
        #self.in_user.set_sensitive(True)
        #self.btnUpdate.set_sensitive(True)
        self.hbxUsername.set_sensitive(True)
        self.icvMain.set_sensitive(True)
        self.btnCancel.set_sensitive(False)
        self.progress.set_fraction(0)
        self.progress.set_text(msg)
        self.downloadStarted = False
        self.current = None
        self.download_queue = []

    def quit(self, *a):
        # XXX: kill all threads!
        self.pd.clear_tmp_folder()
        gtk.main_quit()

def uglyDirtyReadmeHack():
    res = ''
    content = open(os.path.join(current_abs_path, 'README')).read()
    res += content[0]
    for i in range(1, len(content)-1):
        if (content[i-1] != '\n' and content[i] == '\n' and content[i+1].isalnum()):
             res += ' '
        else: res += content[i]
        i += 1
    res += content[-1]
    return res

def downloadLoop(pd):
    while 1:
        if pd.downloadStarted: pd.downloadSelectedAlbums()

class BackToAlbum(object):

    def __init__(self, size):
        self.pixbuf = gtk.icon_theme_get_default().load_icon(gtk.STOCK_GO_UP, 64, 0)
        self.legend = 'Back to albums'

if __name__ == '__main__':
    if '--text' in sys.argv or TEXT_MODE:
        text_mode()
    else:
        pd = PycasaDownloadGUI()

        thread.start_new(downloadLoop, (pd,))
        if sys.platform == 'win32':
            gobject.threads_init()
        else:
            gtk.gdk.threads_init()

        gtk.main()
