#  Copyright 2008, Matias Bordese
#

#  This file is part of PycasaDownload.

#  PycasaDownload is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PycasaDownload is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PycasaDownload. If not, see <http://www.gnu.org/licenses/>.

from DownloadableItem import DownloadableItem
from Photo import Photo
from Thumbnail import Thumbnail

from config import SMALL_THUMB, MEDIUM_THUMB, LARGE_THUMB

import gdata.photos.service

class Album(DownloadableItem):
    """Represent a Picasa album"""

    def __init__(self, albumEntry):
        super(Album, self).__init__(title=albumEntry.title.text)

        self.pws = gdata.photos.service.PhotosService()
        self.albumEntry = albumEntry
        self.numphotos = int(self.albumEntry.numphotos.text)
        self.published = self.albumEntry.published.text[:10]
        self.summary = self.albumEntry.summary.text
        self.thumbnails = [Thumbnail(albumEntry)]
        self.photos = list(self._getPhotosIterator())
        self.path = None

    def _getPhotosIterator(self):
        photos = self.pws.GetFeed(self.albumEntry.GetPhotosUri()).entry
        return (Photo(e) for e in photos)

    def download(self, dest):
        for p in self.photos:
            p.download(dest)

    def getThumbnail(self, size=SMALL_THUMB):
        thumb = self.thumbnails[0]
        thumb.download()
        return thumb
