#  Copyright 2008, Matias Bordese
#

#  This file is part of PycasaDownload.

#  PycasaDownload is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PycasaDownload is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PycasaDownload. If not, see <http://www.gnu.org/licenses/>.

from DownloadableItem import DownloadableItem
from Thumbnail import Thumbnail

from config import SMALL_THUMB, MEDIUM_THUMB, LARGE_THUMB

class Photo(DownloadableItem):
    """Represent a Picasa photo"""

    def __init__(self, photoEntry):
        super(Photo, self).__init__(title=photoEntry.title.text, url=photoEntry.content.src)

        self.photoEntry = photoEntry
        self.weight = int(self.photoEntry.size.text)
        self.width = int(self.photoEntry.width)
        self.height = int(self.photoEntry.height)
        self.published = self.photoEntry.published.text[:10]
        self.summary = self.photoEntry.summary.text
        self.thumbnails = [Thumbnail(photoEntry, size=i) for i in range(0,3)]

    def size(self):
        return (self.width, self.height)

    def getThumbnail(self, size=SMALL_THUMB):
        thumb = self.thumbnails[size]
        thumb.download()
        return thumb
