#  Copyright 2008, Matias Bordese
#

#  This file is part of PycasaDownload.

#  PycasaDownload is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PycasaDownload is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PycasaDownload. If not, see <http://www.gnu.org/licenses/>.

from Album import Album
from Photo import Photo
from Thumbnail import Thumbnail
from config import THUMBNAILS_FOLDER_PATH, SMALL_THUMB, MEDIUM_THUMB, LARGE_THUMB

import os
import sys
from urllib import urlopen
from shutil import rmtree

try:
    import gdata.photos.service
except:
    sys.stderr.write("Google Picasa gdata API not found\n")
    sys.exit(1)

class PycasaDownload:
    """Simple interface to gdata photo api"""

    def __init__(self):
        self.pws = gdata.photos.service.PhotosService()
        self.version = "0.5-beta"
        self.name = "Pycasa Download"
        self.found = 0

        if not os.path.isdir(THUMBNAILS_FOLDER_PATH): os.mkdir(THUMBNAILS_FOLDER_PATH)

    def clear_tmp_folder(self):
        if os.path.isdir(THUMBNAILS_FOLDER_PATH): rmtree(THUMBNAILS_FOLDER_PATH)

    def getAlbumsFromUser(self, user):
        try:
            albums = self.pws.GetUserFeed(user=user).entry
        except gdata.photos.service.GooglePhotosException:
            pass
        else:
            self.found = len(albums)
            return (Album(a) for a in albums)

    def getEntriesFromAlbum(self, album):
        #if not album.photos: album.fetchPhotos(self.pws)
        return album.photos

    def getThumbnailsFromAlbum(self, album, res=SMALL_THUMB):
        #if not album.photos: album.fetchPhotos(self.pws)
        return (photo.thumbnail for photo in album.photos)

    def downloadPhotoFromEntry(self, photo_entry, dest="."):
        photo_entry.download(dest)

