#  Copyright 2008, Matias Bordese
#

#  This file is part of PycasaDownload.

#  PycasaDownload is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PycasaDownload is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PycasaDownload. If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from PycasaDownload import PycasaDownload

def text_mode_step():
        pd = PycasaDownload()

        # get user and albums
        sys.stdout.write("Picasa user: ")
        user = sys.stdin.readline().strip()
        albums = pd.getAlbumsFromUser(user)
        if not albums:
            sys.stdout.write("No albums found\n")
            return

        # show album list
        i = 1
        sys.stdout.write("\t%d albums found\n" % len(albums))
        for a in albums:
            sys.stdout.write("%d - %s (%s)\n" % (i, a.title.text, a.numphotos.text))
            i += 1
        sys.stdout.write("0 - None\n")

        # get user options: album, destination
        sys.stdout.write("Which albums do you want to download? [numbers, comma separated]: ")
        try:
            a = sys.stdin.readline()[:-1]
            numbers = [int(i) for i in a.split(',')]
        except:
            sys.stderr.write("Invalid option\n")
            sys.exit(1)
        if a == '0':
            return

        sys.stdout.write("Destination directory [empty for current]: ")
        dest_base = sys.stdin.readline()[:-1]

        # Download loop
        for n in numbers:
            if n < 0 or n > len(albums):
                sys.stdout.write("Invalid index: %d. Skipped\n" % n)
                continue
            album = albums[n - 1]
            if not dest_base:
                dest_base = '.'
            dest = dest_base + "/%s" % album.title.text
            try:
                os.mkdir(dest)
            except:
                sys.stderr.write("Invalid destination directory\n")
                sys.exit(1)

            photos = pd.getEntriesFromAlbum(album)
            count = len(photos)
            sys.stdout.write("Downloading %s\n" % album.title.text)
            for p in photos:
                sys.stdout.write(".")
                sys.stdout.flush()
                pd.downloadPhotoFromEntry(p, dest)
            sys.stdout.write("\n")
        sys.stdout.write("Successfully download\n")


def text_mode():
    sys.stdout.write("\n\nPycasaDownload 0.2\n(c) 2008, Matias Bordese\n\n")
    keep_downloading = True
    while keep_downloading:
        text_mode_step()
        sys.stdout.write("Enter a new username? [Y/N]: ")
        ans = sys.stdin.readline()[:-1]
        keep_downloading =  ans == 'Y' or ans == 'y'

if __name__ == "__main__":
    text_mode()
